##############################################
# R script for RSiena analysis of Coleman's
# longitudinal adolescents data. 
# version of 4 February 2017
# written by Christian Steglich
##############################################

# In the fall of 1957 and in the spring of
# 1958, James Coleman asked the 73 boys of a
# small high school in Illinois, "What
# fellows  here in school do you go around
# with most often?"
# The research is reported in:
# Coleman, James S. "The Adolescent Society:
#   The Social Life of the Teenager and its
#   Impact on Education." Glencoe, IL:
#   Free Press (1961).
# The data set is published in:
# Coleman, James S. "Introduction to
#   Mathematical Sociology." London:
#   The Free Press of Glencoe (1964).

##############################################

# load RSiena commands to the R namespace:
library(RSiena)

# check software version:
packageVersion("RSiena")
# always good to know

# load data set:
load("Coleman.RData")

# network change between observations,
table(coleman[[1]],coleman[[2]],useNA='always')
# Jaccard index is 0.38, Hamming distance is 227.

# create data object for the analysis:
coleman.data <- sienaDataCreate(
	goAround=sienaDependent(
		array(c(coleman[[1]],coleman[[2]]),dim=c(73,73,2))
		)
	)

# create model specification object:
first.model <- getEffects(coleman.data)

# check what't possible to include:
effectsDocumentation(first.model)

# include some typical effects:
first.model <- includeEffects(first.model,
	transTrip,transRecTrip,cycle3)
first.model

# specify the options for the estimation algorithm:
estimation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	cond=FALSE,seed=1234567)

# estimate the model:
first.results <- siena07(estimation.options,data=coleman.data,
	effects=first.model,batch=FALSE,verbose=FALSE)
first.results # has converged (in RSiena 1.1.304)

# Now let's move towards goodness-of-fit inspection...

# specify the options for the simulation algorithm:
simulation.options <- sienaAlgorithmCreate(useStdInits=FALSE,cond=FALSE,
	n3=250,nsub=0,seed=1234567)

# add simulated networks to model results:
first.sims <- siena07(simulation.options,data=coleman.data,
	effects=first.model,batch=FALSE,verbose=FALSE,returnDeps=TRUE,
	prevAns=first.results)

# Now assess goodness of fit on several dimensions:

	# goodness of fit for indegree distribution:
	gof1.indegrees <- sienaGOF(first.sims,IndegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof1.indegrees
	plot(gof1.indegrees) # looks good enough already!

	# goodness of fit for outdegree distribution:
	gof1.outdegrees <- sienaGOF(first.sims,OutdegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof1.outdegrees
	plot(gof1.outdegrees) # looks bad... strange peak at outdegree 5

	# import custom-made fit functions:
	source("sienaGOF-auxiliary.R")
	# note that they require packages 'network' and 'sna'

	# goodness of fit for triad census:
	gof1.triads <- sienaGOF(first.sims,TriadCensus,
		verbose=TRUE,varName="goAround")
	gof1.triads
	plot(gof1.triads,center=TRUE,scale=TRUE) # looks bad for triad 111U

	# goodness of fit for geodesic distances:
	gof1.geodesic <- sienaGOF(first.sims,GeodesicDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof1.geodesic
	plot(gof1.geodesic) # looks bad (as usual for geodesics)

# CONCLUSION from the above: do s.th about the outdegree
# distribution, and try to specifically model triad 111U:
second.model <- includeEffects(first.model,reciAct)

# estimate the model (two runs needed to obtain convergence):
second.results <- siena07(estimation.options,data=coleman.data,
	effects=second.model,batch=FALSE,verbose=FALSE,
	prevAns=first.results)
second.results # has converged (in RSiena 1.1.304)

# add simulated networks to model results:
second.sims <- siena07(simulation.options,data=coleman.data,
	effects=second.model,batch=FALSE,verbose=FALSE,returnDeps=TRUE,
	prevAns=second.results)

# Now inspect if goodness-of-fit has improved...

	# goodness of fit for indegree distribution:
	gof2.indegrees <- sienaGOF(second.sims,IndegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof2.indegrees
	plot(gof2.indegrees,main='second model') # still good

	# goodness of fit for outdegree distribution:
	gof2.outdegrees <- sienaGOF(second.sims,OutdegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof2.outdegrees
	plot(gof2.outdegrees) # improved a bit, but still bad

	# goodness of fit for triad census:
	gof2.triads <- sienaGOF(second.sims,TriadCensus,
		verbose=TRUE,varName="goAround")
	gof2.triads
	plot(gof2.triads,center=TRUE,scale=TRUE) # looks more reasonable for triad 111U

	# goodness of fit for geodesic distances:
	gof2.geodesic <- sienaGOF(second.sims,GeodesicDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof2.geodesic
	plot(gof2.geodesic) # also here considerable improvement

# based on triad census fit, improvement on
# triad 111D now is a good candidate for making the
# overall fit better...
third.model <- includeEffects(second.model,reciPop)

# estimate the model:
third.results <- siena07(estimation.options,data=coleman.data,
	effects=third.model,batch=FALSE,verbose=FALSE,
	prevAns=second.results)
third.results # has converged (in RSiena 1.1.304)

# add simulated networks to model results:
third.sims <- siena07(simulation.options,data=coleman.data,
	effects=third.model,batch=FALSE,verbose=FALSE,returnDeps=TRUE,
	prevAns=third.results)

# Inspect if goodness-of-fit has improved...

	# goodness of fit for indegree distribution:
	gof3.indegrees <- sienaGOF(third.sims,IndegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof3.indegrees
	plot(gof3.indegrees) # now REALLY good

	# goodness of fit for outdegree distribution:
	gof3.outdegrees <- sienaGOF(third.sims,OutdegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof3.outdegrees
	plot(gof3.outdegrees) # again improved a bit, but still bad

	# goodness of fit for triad census:
	gof3.triads <- sienaGOF(third.sims,TriadCensus,
		verbose=TRUE,varName="goAround")
	gof3.triads
	plot(gof3.triads,center=TRUE,scale=TRUE) # looks more reasonable also for triad 111D;

	# goodness of fit for geodesic distances:
	gof3.geodesic <- sienaGOF(third.sims,GeodesicDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof3.geodesic
	plot(gof3.geodesic) # again some improvement

# let us now model the idiosyncratic kinks in the outdegree distribution:
# (non-linear outdegree efects like outAct don't help here)
fourth.model <- includeEffects(third.model,outTrunc,outTrunc2)
fourth.model <- setEffect(fourth.model,outTrunc,parameter=4)
fourth.model <- setEffect(fourth.model,outTrunc2,parameter=5)

# estimate the model:
fourth.results <- siena07(estimation.options,data=coleman.data,
	effects=fourth.model,batch=FALSE,verbose=FALSE,
	prevAns=third.results)
fourth.results # has NOT converged (in RSiena 1.1.304), another run:
fourth.results <- siena07(estimation.options,data=coleman.data,
	effects=fourth.model,batch=FALSE,verbose=FALSE,
	prevAns=fourth.results)
fourth.results # has NOT converged (in RSiena 1.1.304), another run:
fourth.results <- siena07(estimation.options,data=coleman.data,
	effects=fourth.model,batch=FALSE,verbose=FALSE,
	prevAns=fourth.results)
fourth.results # has converged (in RSiena 1.1.304)

# add simulated networks to model results:
fourth.sims <- siena07(simulation.options,data=coleman.data,
	effects=fourth.model,batch=FALSE,verbose=FALSE,returnDeps=TRUE,
	prevAns=fourth.results)

# ...and inspect how goodness-of-fit has changed...

	# goodness of fit for indegree distribution:
	gof4.indegrees <- sienaGOF(fourth.sims,IndegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof4.indegrees
	plot(gof4.indegrees) # still very okay

	# goodness of fit for outdegree distribution:
	gof4.outdegrees <- sienaGOF(fourth.sims,OutdegreeDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof4.outdegrees
	plot(gof4.outdegrees) # finally somewhat okay, but still not good!

	# goodness of fit for triad census & geodesic distances:
	gof4.triads <- sienaGOF(fourth.sims,TriadCensus,
		verbose=TRUE,varName="goAround")
	gof4.triads
	plot(gof4.triads,center=TRUE,scale=TRUE) # this one fits extremely well now!

	gof4.geodesic <- sienaGOF(fourth.sims,GeodesicDistribution,
		verbose=TRUE,varName="goAround",cumulative=FALSE)
	gof4.geodesic
	plot(gof4.geodesic) # again an improvement

# Conclusion: we now have a model that is idiosyncratic and probably
# overfitting the data, but still we do not have perfect fit in the
# problematic outdegree distribution... at some point we need to
# concede defeat and become content with what was possible while still
# making non-idiosyncrtatic sense. That would be model three above.


# save results if needed later:
save.image("goodnes of fit lab results.RData")